from ..config.config import BASE_SYSTEM_TEXT, API_KEY, MODELS_NAME
import random

class History:
    def __init__(self, system_prompt: str = BASE_SYSTEM_TEXT):
        self.history = []
        if system_prompt is not None:
            self.add_to_history("system", system_prompt)

    def add_to_history(self, role: str, content: str):
        self.history.append({
            "role": role,
            "content": content
        })


class ParentModel():
    def __init__(self, api_key=API_KEY, history: History = History()):
        self.history = history
        self.api_key = api_key

        self.current_model_index = 0
        models_copy = MODELS_NAME.copy()
        random.shuffle(models_copy)
        self.regen_models_name = models_copy


class Model(ParentModel):

    def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        return "answer"
    
    def get_answer_without_history(self, prompt: list[dict[str, str]]) -> str:
        return self.generate_answer(with_history=False, prompt=prompt)

    def get_answer(self, prompt: str) -> str:
        self.history.add_to_history("user", prompt)
        model_answer = self.generate_answer()
        self.history.add_to_history("assistant", model_answer)
        return model_answer
    
class AsyncModel(ParentModel):

    async def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        return "answer"
    
    async def get_answer_without_history(self, prompt: list[dict[str, str]]) -> str:
        return await self.generate_answer(with_history=False, prompt=prompt)

    async def get_answer(self, prompt: str) -> str:
        self.history.add_to_history("user", prompt)
        model_answer = await self.generate_answer()
        self.history.add_to_history("assistant", model_answer)
        return model_answer





from .preprocessor.spliter import split_data, gen_doc_parts, async_gen_doc_parts
from .preprocessor.compressor import compress_to_one, generate_discribtions_for_code
from .preprocessor.postprocess import get_introdaction, get_all_html_links, get_links_intro
from .engine.models.gpt_model import AsyncGPTModel, GPTModel
from .engine.models.model import Model, AsyncModel
import os
from .preprocessor.code_mix import CodeMix
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn, TimeRemainingColumn
import asyncio
from .factory.base_factory import DocFactory
from .factory.modules.intro import IntroLinks, IntroText
from .factory.modules.general_modules import CustomModule
from .ui.progress_base import BaseProgress, LibProgress
from .preprocessor.settings import ProjectSettings


class Manager:
    CACHE_FOLDER_NAME = ".auto_doc_cache"

    FILE_NAMES = {
        "code_mix": "code_mix.txt",
        "global_info": "global_info.md",
        "output_doc": "output_doc.md"
    }


    def __init__(self, project_directory: str, project_settings: ProjectSettings,
                 sync_model: Model = None, async_model: AsyncModel = None,
                  ignore_files: list = [], language: str = "en", 
                  progress_bar: BaseProgress = BaseProgress()):
        self.project_directory = project_directory
        self.ignore_files = ignore_files
        self.progress_bar = progress_bar
        self.language = language
        self.project_settings = project_settings

        self.sync_model = sync_model
        self.async_model = async_model

        cache_path = os.path.join(self.project_directory, self.CACHE_FOLDER_NAME)

        if not os.path.isdir(cache_path):
            os.mkdir(cache_path)

    def read_file_by_file_key(self, file_key: str):
        with open(self.get_file_path(file_key), "r", encoding="utf-8") as file:
            data = file.read()
        return data

    def get_file_path(self, file_key: str):
        return os.path.join(self.project_directory, self.CACHE_FOLDER_NAME, self.FILE_NAMES.get(file_key))

    def generate_code_file(self):
        cm = CodeMix(self.project_directory, self.ignore_files)
        cm.build_repo_content(self.get_file_path("code_mix"))
        self.progress_bar.update_task()

    def generate_global_info_file(self, max_symbols=10_000, use_async: bool = False):
        full_code_mix = self.read_file_by_file_key("code_mix")

        splited_data = split_data(full_code_mix, max_symbols)
        curr_model = self.async_model if use_async else self.sync_model
        result = compress_to_one(splited_data, curr_model, self.project_settings, 2, progress_bar=self.progress_bar, use_async=use_async)
        with open(self.get_file_path("global_info"), "w", encoding="utf-8") as file:
            file.write(result)

        self.progress_bar.update_task()
        

    def generete_doc_parts(self, max_symbols=5_000, use_async: bool = False):
        global_info = self.read_file_by_file_key("global_info")
        full_code_mix = self.read_file_by_file_key("code_mix")

        if use_async:
            result = asyncio.run(async_gen_doc_parts(full_code_mix, global_info, max_symbols, self.async_model, self.language, self.progress_bar))
        else:
            result = gen_doc_parts(full_code_mix, global_info, max_symbols, self.sync_model, self.language, self.progress_bar)

        with open(self.get_file_path("output_doc"), "w", encoding="utf-8") as file:
            file.write(result)
        
        self.progress_bar.update_task()

    def factory_generate_doc(self, doc_factory: DocFactory):
        global_info = self.read_file_by_file_key("global_info")
        curr_doc = self.read_file_by_file_key("output_doc")
        code_mix = self.read_file_by_file_key("code_mix")

        info = {
            "language": self.language,
            "global_data": global_info,
            "full_data": curr_doc,
            "code_mix": code_mix
        }
        result = doc_factory.generate_doc(info, self.sync_model, self.progress_bar)

        new_data = f"{result} \n\n{curr_doc}"

        with open(self.get_file_path("output_doc"), "w", encoding="utf-8") as file:
            file.write(new_data)

        self.progress_bar.update_task()
        

        
from .engine.config.config import API_KEY

if __name__ == "__main__":
    ignore_list = [
        "*.pyo", "*.pyd", "*.pdb", "*.pkl", "*.log", "*.sqlite3", "*.db", "data",
        "venv", "env", ".venv", ".env", ".vscode", ".idea", "*.iml", ".gitignore", ".ruff_cache", ".auto_doc_cache",
        "*.pyc", "__pycache__", ".git", ".coverage", "htmlcov", "migrations", "*.md", "static", "staticfiles", ".mypy_cache"
    ]

    sync_model = GPTModel(API_KEY)
    async_model = AsyncGPTModel(API_KEY)

    with Progress(
        SpinnerColumn(),          
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),               
        TaskProgressColumn(),     
    ) as progress:
        project_settings = ProjectSettings("Auto Doc Generator")
        project_settings.add_info(
            "global idea",
            """This project was created to help developers make documentations for them projects"""
        )
        manager = Manager(r"C:\Users\sinic\OneDrive\Документы\GitHub\ADG", 
                        project_settings,
                        sync_model=sync_model,
                        async_model=async_model,
                        ignore_files=ignore_list, 
                        progress_bar=LibProgress(progress), 
                        language="en")

        # manager.generate_code_file()
        # manager.generate_global_info_file(use_async=True, max_symbols=5000)
        # manager.generete_doc_parts(use_async=True, max_symbols=4000)
        # manager.factory_generate_doc(
        #     DocFactory(
        #         CustomModule("how to use Manager class what parameters i need to give. give full example of usege"),
        #         CustomModule("how to use Module and create your own module. give full example of usege ")
        #     )
        # )
        
        manager.factory_generate_doc(
            DocFactory(
                IntroLinks(),
                # IntroText(),
            )
        )






from abc import ABC, abstractmethod
from ..engine.models.model import Model, AsyncModel
from ..ui.progress_base import BaseProgress

class BaseModule(ABC):
    def __init__(self):
        pass
    
    @abstractmethod
    def generate(self, info: dict, model: Model):
        ...




class DocFactory:
    def __init__(self, *modules):
        self.modules: list[BaseModule] = modules

    def generate_doc(self, info: dict, model: Model, progress: BaseProgress):
        output = ""
        progress.create_new_subtask("Generate parts", len(self.modules))
        for module in self.modules:
            module_result = module.generate(info, model)
            output += module_result + "\n\n"
            progress.update_task()
        progress.remove_subtask()

        return output

if __name__ == "__main__":
    DocFactory(BaseModule(), BaseModule())


from ..engine.config.config import BASE_SETTINGS_PROMPT

class ProjectSettings:
    def __init__(self, project_name: str):
        self.project_name = project_name
        self.info = {}

    def add_info(self, key, value):
        self.info[key] = value

    @property
    def prompt(self):
        result = ""
        result += BASE_SETTINGS_PROMPT
        result += f"Project Name: {self.project_name} \n"
        for key in self.info:
            result += f"{key}: {self.info[key]} \n"
        
        return result
    



import os
from pathlib import Path
import fnmatch

class CodeMix:
    def __init__(self, root_dir=".", ignore_patterns=None):
        self.root_dir = Path(root_dir).resolve()
        self.ignore_patterns = ignore_patterns or []

    def should_ignore(self, path: str) -> bool:
        relative_path = path.relative_to(self.root_dir)
        path_str = str(relative_path)
        
        for pattern in self.ignore_patterns:
            if fnmatch.fnmatch(path_str, pattern) or \
               fnmatch.fnmatch(os.path.basename(path_str), pattern) or \
               any(fnmatch.fnmatch(p, pattern) for p in path.parts):
                return True
        return False

    def build_repo_content(self, output_file="repomix-output.txt"):
        with open(output_file, "w", encoding="utf-8") as out:
            out.write("Repository Structure:\n")
            for path in sorted(self.root_dir.rglob("*")):
                if self.should_ignore(path):
                    continue
                depth = len(path.relative_to(self.root_dir).parts)
                indent = "  " * (depth - 1)
                out.write(f"{indent}{path.name}/\n" if path.is_dir() else f"{indent}{path.name}\n")
            
            out.write("\n" + "="*20 + "\n\n")

            for path in sorted(self.root_dir.rglob("*")):
                if path.is_file() and not self.should_ignore(path):
                    try:
                        relative_path = path.relative_to(self.root_dir)
                        out.write(f'<file path="{relative_path}">\n')
                        out.write(path.read_text(encoding="utf-8", errors="ignore"))
                        out.write(f"\n</file>\n\n")
                    except Exception as e:
                        out.write(f"Error reading {path}: {e}\n")

ignore_list = [
    "*.pyo", "*.pyd", "*.pdb", "*.pkl", "*.log", "*.sqlite3", "*.db",
    "venv", "env", ".venv", ".env", ".vscode", ".idea", "*.iml", ".gitignore", ".ruff_cache",
    "*.pyc", "__pycache__", ".git", ".coverage", "htmlcov", "migrations", "*.md", "static", "staticfiles", ".mypy_cache"
]


if __name__ == "__main__":
    packer = CodeMix(root_dir=r"C:\Users\huina\Python Projects\Kwork\ClickerProject\ClickerApp", ignore_patterns=ignore_list)
    packer.build_repo_content("codemix.txt")
    print("Файл успешно создан!")



from ..engine.models.gpt_model import GPTModel
from ..engine.models.model import Model 
from ..engine.config.config import BASE_INTRODACTION_CREATE_TEXT, BASE_INTRO_CREATE
import re
import unicodedata

def generate_markdown_anchor(header: str) -> str:
    anchor = header.lower()
    anchor = unicodedata.normalize('NFKC', anchor)
    anchor = anchor.replace(' ', '-')
    anchor = re.sub(r'[^a-z0-9\-_]', '', anchor)

    anchor = re.sub(r'-+', '-', anchor)
    anchor = anchor.strip('-')
    
    return f"#{anchor}"

def get_all_topics(data: str) -> list[str]:
    topics = []
    curr_shift_index = 0
    while True:
        curr_index = data.find("\n## ", curr_shift_index)
        if curr_index == -1:
            break
        
        curr_shift_index = data.find("\n", curr_index + 3)
        topics.append(data[curr_index + 4:curr_shift_index])

    links = [generate_markdown_anchor(el) for el in topics]
    return topics, links 

def get_all_html_links(data: str) -> list[str]:
    links = []
    curr_shift_index = 0
    while True:
        curr_index = data.find("<a name=", curr_shift_index)
        if curr_index == -1:
            break

        curr_shift_index = data.find("</a>", curr_index + 3)
        curr_result = data[curr_index + 9:curr_shift_index - 2]
        if len(curr_result) > 25:
            continue
        links.append("#" + curr_result)

    return links

def get_links_intro(links: list[str], model: Model, language: str = "en"):
    prompt = [
        {
            "role": "system",
            "content": f"For the following task use language {language}"
        },
        {
            "role": "system",
            "content": BASE_INTRODACTION_CREATE_TEXT
        },
        {
            "role": "user",
            "content": str(links)
        }
    ]
    intro_links = model.get_answer_without_history(prompt=prompt)
    return intro_links

def get_introdaction(global_data: str, model: Model, language: str = "en") -> str:
    prompt = [
        {
            "role": "system",
            "content": f"For the following task use language {language}"
        },
        {
            "role": "system",
            "content": BASE_INTRO_CREATE
        },
        {
            "role": "user",
            "content": global_data
        }
    ]

    intro = model.get_answer_without_history(prompt=prompt)

    return intro

def generete_custom_discription(splited_data: str, model: Model, custom_description: str, language: str = "en") -> str:

    for sp_data in splited_data:

        prompt = [
            {
                "role": "system",
                "content": f"For the following task use language {language}"
            },
            {
                "role": "system",
                "content": f"Act as a precise Technical Analyst. You will be provided with specific code or documentation. Your task is to describe or extract information based ONLY on the provided context. And make title and link <a name='your_title'> </a> format"
            },
            {
                "role": "system",
                "content": f"### Context: {sp_data}"
            },
            {"role": "system",
            "content": """### Strict Rules:
                1. Use ONLY the provided Context to answer. 
                2. If the requested information is not explicitly mentioned in the Context, or if you don't know the answer based on the provided data, respond with an empty string ("") or simply say "No information found". 
                3. DO NOT use external knowledge or invent any logic that is not present in the text.
                4. Do not provide any introductory or concluding remarks. If there is no info, output must be empty.
                5. If you dont have any info about it return just !noinfo"""
            },
            {
                "role": "user",
                "content": f"### Task to discribe: {custom_description}"
            }
        ]

        result = model.get_answer_without_history(prompt=prompt)
        if (result.find("!noinfo") == -1 and result.find("No information found") == -1) or result.find("!noinfo") > 30:
            break
        result = ""
    return result


from .model import Model, AsyncModel, API_KEY, History
from groq import Groq, AsyncGroq
from ..config.config import MODELS_NAME


class AsyncGPTModel(AsyncModel):
    def __init__(self, api_key=API_KEY, history = History()):
        super().__init__(api_key, history)
        self.client = AsyncGroq(api_key=self.api_key)

    async def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        
        if with_history:
            messages = self.history.history
        else:
            messages = prompt
        
        chat_completion = None
        
        while True:
            model_name = self.regen_models_name[self.current_model_index]
            try:
                chat_completion = await self.client.chat.completions.create(
                    messages=messages,
                    model=model_name,
                    # temperature=0.3,
                )
                break
            except Exception as e:
                print(e)
                self.current_model_index = 0 if self.current_model_index + 1 >= len(self.regen_models_name) else self.current_model_index + 1
                    

        if chat_completion is None:
            raise Exception("all models do not work")

        return chat_completion.choices[0].message.content

class GPTModel(Model):
    def __init__(self, api_key=API_KEY, history = History()):
        super().__init__(api_key, history)
        self.client = Groq(api_key=self.api_key)

    def generate_answer(self, with_history: bool = True, prompt: str = None) -> str:
        
        if with_history:
            messages = self.history.history
        else:
            messages = prompt
        
        chat_completion = None

        while True:
            model_name = self.regen_models_name[self.current_model_index]
            try:
                chat_completion = self.client.chat.completions.create(
                    messages=messages,
                    model=model_name,

                )
                break
            except Exception as e:
                print(e)
                self.current_model_index = 0 if self.current_model_index + 1 >= len(self.regen_models_name) else self.current_model_index + 1

        if chat_completion is None:
            raise Exception("all models do not work")

        return chat_completion.choices[0].message.content


from ..engine.models.gpt_model import GPTModel, AsyncGPTModel, Model, AsyncModel
from ..engine.config.config import get_BASE_COMPRESS_TEXT
import math
import asyncio
from ..ui.progress_base import BaseProgress
from .settings import ProjectSettings


def compress(data: str, project_settings: ProjectSettings, model: Model, compress_power) -> str:
    prompt = [
        {
            "role": "system",
            "content": project_settings.prompt
        },
        {
            "role": "system",
            "content": get_BASE_COMPRESS_TEXT(10000, compress_power)
        },
        {
            "role": "user",
            "content": data
        }
    ]
    answer = model.get_answer_without_history(prompt=prompt)

    return answer


def compress_and_compare(data: list, model: Model, project_settings: ProjectSettings, compress_power: int = 4, progress_bar: BaseProgress = BaseProgress()) -> list:
    compress_and_compare_data = ["" for i in range(math.ceil(len(data) / compress_power))]
    progress_bar.create_new_subtask(f"Compare all files", len(data))
    for i, el in enumerate(data):
        curr_index = i // compress_power
        compress_and_compare_data[curr_index] += compress(el, project_settings, model, compress_power) + "\n"
        progress_bar.update_task()

    progress_bar.remove_subtask()

    return compress_and_compare_data

async def async_compress(data: str,  project_settings: ProjectSettings, model: AsyncModel, compress_power, semaphore, progress_bar: BaseProgress) -> str:
    
    async with semaphore:
        prompt = [
            {
                "role": "system",
                "content": project_settings.prompt
            },
            {
                "role": "system",
                "content": get_BASE_COMPRESS_TEXT(10000, compress_power)
            },
            {
                "role": "user",
                "content": data
            }
        ]
        answer = await model.get_answer_without_history(prompt=prompt)
        progress_bar.update_task()
        return answer

async def async_compress_and_compare(data: list, model: AsyncModel, project_settings: ProjectSettings, compress_power: int = 4, progress_bar: BaseProgress = BaseProgress()) -> list:
    semaphore = asyncio.Semaphore(4)
    tasks = []
    progress_bar.create_new_subtask(f"Compare all files (async)", len(data))

    for el in data:
        tasks.append(async_compress(el, project_settings, model, compress_power, semaphore, progress_bar))
    
    compressed_elements = await asyncio.gather(*tasks)
    
    final_data = []
    chunk_size = compress_power
    for i in range(0, len(compressed_elements), chunk_size):
        chunk = compressed_elements[i : i + chunk_size]
        final_data.append("\n".join(chunk) + "\n")
    
    progress_bar.remove_subtask()
        
    return final_data

def compress_to_one(data: list, model: Model, project_settings: ProjectSettings, compress_power: int = 4, use_async: bool = False, progress_bar: BaseProgress = BaseProgress()):
    count_of_iter = 0
    while len(data) > 1:
        new_compress_power = compress_power
        if (len(data) < compress_power + 1):
            new_compress_power = 2
        
        if use_async:
            data = asyncio.run(async_compress_and_compare(data, model, project_settings, new_compress_power, progress_bar=progress_bar))
        else:
            data = compress_and_compare(data, model, project_settings, new_compress_power, progress_bar=progress_bar)
        count_of_iter += 1


    return data[0]



def generate_discribtions_for_code(data: list, model: Model, project_settings: ProjectSettings, progress_bar: BaseProgress = BaseProgress()) -> list:
    describtions = []
    progress_bar.create_new_subtask("Generate describtions for code files", len(data))
    for code in data:
        prompt = [
            {
                "role": "system",
                "content": """### Instructions:
1. Identify Main Components: Determine what classes, functions, or modules are meant to be used by an external developer.
2. Description: Explain the purpose of each main component (e.g., "What is the Manager class for?").
3. Parameters & Types: Detail every parameter required for initialization and method calls, including their expected data types and default values.
4. Usage Example: Provide a COMPLETE, "copy-pasteable" code example showing:
   - Proper initialization.
   - Calling key methods.
   - Handling expected outputs or errors.

### Strict Rules:
- Base your guide ONLY on the provided code. 
- If the code is incomplete or a certain class/method is missing, do not hallucinate its logic—simply state that information is unavailable.
- Use Markdown for code blocks and bold text for parameter names.
- If no usable logic is found in the code, respond with an empty string ("")."""
            },
            {
                "role": "user",
                "content": f"CONTEXT: {code}"
            }
        ]
        answer = model.get_answer_without_history(prompt=prompt)
        describtions.append(answer)
        progress_bar.update_task()
    
    progress_bar.remove_subtask()
    return describtions







from ..engine.models.gpt_model import GPTModel, AsyncGPTModel, AsyncModel, Model
from ..engine.config.config import BASE_PART_COMPLITE_TEXT
import asyncio
from ..ui.progress_base import BaseProgress


def split_data(data: str, max_symbols: int) -> list[str]:

    split_objects = []

    splited_by_files = data.split("</file>")
    

    while True:
        have_to_change = False
        for i, el in enumerate(splited_by_files):
            if len(el) > max_symbols * 1.5:
                splited_by_files.insert(i+1, el[i][int(max_symbols / 2):])
                splited_by_files[i] = el[i][:int(max_symbols / 2)]
                have_to_change = True

        if have_to_change == False:
            break

            


    curr_index = 0
    for el in splited_by_files:
        if len(split_objects) - 1 < curr_index:
            split_objects.append("")

        if len(split_objects[curr_index]) + len(el) > max_symbols * 1.25:
            curr_index += 1
            split_objects.append(el)
            continue

        split_objects[curr_index] += "\n" + el

    return split_objects

def write_docs_by_parts(part: str, model: Model, global_info: str, prev_info: str = None, language: str = "en"):
    prompt = [
            {
                "role": "system",
                "content": f"For the following task use language {language}"
            },
            {
                "role": "system",
                "content": BASE_PART_COMPLITE_TEXT
            },
            # {
            #     "role": "system",
            #     "content": global_info
            # },
            {
                "role": "user",
                "content": part
            }
    ]

    if prev_info is not None:
        prompt.append({
                "role": "system",
                "content": f"it is last part of documentation that you have write before{prev_info}"
            })

    prompt.append({
                "role": "user",
                "content": part
            })
    
    answer: str = model.get_answer_without_history(prompt=prompt)
    temp_answer = answer.removeprefix("```")
    if answer == temp_answer:
        return answer

    answer = temp_answer.removesuffix("```")
    return answer

async def async_write_docs_by_parts(part: str, async_model: AsyncModel, global_info: str, semaphore, prev_info: str = None, language: str = "en", update_progress = None):

    async with semaphore:

        prompt = [
            {
                "role": "system",
                "content": f"For the following task use language {language}"
            },
            {
                "role": "system",
                "content": BASE_PART_COMPLITE_TEXT
            },
            # {
                # "role": "system",
                # "content": global_info
            # },
            {
                "role": "user",
                "content": part
            }
        ]

        if prev_info is not None:
            prompt.append({
                    "role": "system",
                    "content": f"it is last part of documentation that you have write before{prev_info}"
                })

        prompt.append({
                    "role": "user",
                    "content": part
                })
        answer: str = await async_model.get_answer_without_history(prompt=prompt)

        if update_progress is not None:
            update_progress()


        temp_answer = answer.removeprefix("```")
        if answer == temp_answer:
            return answer

        answer = temp_answer.removesuffix("```")
        return answer


def gen_doc_parts(full_code_mix, global_info, max_symbols, model: Model, language, progress_bar: BaseProgress):
    splited_data = split_data(full_code_mix, max_symbols)
    result = None

    progress_bar.create_new_subtask(f"Generete doc parts", total_len=len(splited_data))
    
    all_result = ""
    for el in splited_data:
        result = write_docs_by_parts(el, model, global_info, result, language)
        all_result += result
        all_result += "\n\n"

        result = result[len(result) - 3000:]
        progress_bar.update_task()
    
    progress_bar.remove_subtask()

    return all_result

async def async_gen_doc_parts(full_code_mix, global_info, max_symbols, model: AsyncModel, language, progress_bar: BaseProgress):
    splited_data = split_data(full_code_mix, max_symbols)
    progress_bar.create_new_subtask(f"Generete doc parts (async)", len(splited_data))

    semaphore = asyncio.Semaphore(4)

    tasks = []
    for el in splited_data:
        tasks.append(async_write_docs_by_parts(part=el, async_model=model, global_info=global_info, semaphore=semaphore, language=language, update_progress=lambda: progress_bar.update_task()))

    gen_parts = await asyncio.gather(*tasks)
    result = ""
    for el in gen_parts:
        result += el
        result += "\n\n"

    progress_bar.remove_subtask()


    return result


from rich.progress import Progress


class BaseProgress:
    def __init__(self):
        pass

    def create_new_subtask(self, name: str, total_len: int):
        ...

    def update_task(self):
        ...

    def remove_subtask(self):
        ...

class LibProgress(BaseProgress):
    
    def __init__(self, progress: Progress, total=4):
        super().__init__()
        self.progress = progress
        self._base_task = self.progress.add_task("General progress", total=total)
        self._cur_sub_task = None

    def create_new_subtask(self, name, total_len):
        self._cur_sub_task = self.progress.add_task(name, total=total_len)

    def update_task(self):
        if self._cur_sub_task is None:
            self.progress.update(self._base_task, advance=1)
        else:
            self.progress.update(self._cur_sub_task, advance=1)

    def remove_subtask(self):
        if self._cur_sub_task is not None:
            self._cur_sub_task = None
